@@ -1,8 +1,13 @@
 from dataclasses import replace
 import os
+from time import time
 from typing import Any
 import requests
 import json
+from clint.textui import progress
+import requests
+import shutil
+from tqdm.auto import tqdm
 
 class ManageFiles():
     def __init__(self, fms_config_variables):
@@ -22,37 +27,56 @@ class ManageFiles():
             # Setting the url for the file by combining FMS API base url and the file key
             
             # Sending get request to FMS API 
-            with requests.get(url, stream=True) as response:
-                # Retrun error if happened during HTTP request
+            # with requests.get(url, stream=True) as response:
+            #     # Retrun error if happened during HTTP request
                 
-                # for i in response.iter_lines():
-                #         download_response += str(i)
-                response.raise_for_status()
+            #     # for i in response.iter_lines():
+            #     #         download_response += str(i)
+            #     response.raise_for_status()
                 
-                # Getting file extension from request headers
-                file_extention = response.headers['x-ext'].replace('.','')
+            #     # Getting file extension from request headers
+            #     file_extention = response.headers['x-ext'].replace('.','')
+            #     # Setting file storage location using file key as the name of the saved file
+            #     file_location = os.path.join(base_path,f"{file_key}.{file_extention}")
+            #     # Writing recieved chuncks from the get request
+            #     with open(file_location, 'wb') as file:
+            #         # Itterating through the recieved chuncks to be writtien to the created file
+            #         for chunk in response.iter_content(chunk_size=1024): 
+            #             file.write(chunk)
+            #     # Closing get request after finishing download
+                
+            #     response.close()
+                
+        
+            with requests.get(url, stream=True) as r:
+                file_extention = r.headers['x-ext'].replace('.','')
                 # Setting file storage location using file key as the name of the saved file
                 file_location = os.path.join(base_path,f"{file_key}.{file_extention}")
-                # Writing recieved chuncks from the get request
-                with open(file_location, 'wb') as file:
-                    # Itterating through the recieved chuncks to be writtien to the created file
-                    for chunk in response.iter_content(chunk_size=1024): 
-                        file.write(chunk)
-                # Closing get request after finishing download
+                # check header to get content length, in bytes
+                total_length = int(r.headers.get("Content-Length"))
                 
-                response.close()
+                # implement progress bar via tqdm
+                with tqdm.wrapattr(r.raw, "read", total=total_length, desc="")as raw:
                 
+                    # save the output to a file
+                    with open(file_location, 'wb')as output:
+                        shutil.copyfileobj(raw, output)
                 
             # Return success message
             return 'File Downloaded Successfully', file_key, file_extention, True
         except Exception as err:
+            return self.get_response(url)
+
+    def get_response(self, url):
+        try:
             response = b''
             for response_bytes in requests.get(url).iter_lines():
                         response += response_bytes
             response = json.loads(response.decode('utf8').replace("'", '"'))
             response = response["message"]
             return response, 'Error', 'Error', False
-
+        except:
+            return "Error While Downloading File", 'Error', 'Error', False
     # This function is used to upload files to FMS API 
     # it takes file key(to be sent to FMS API), name(to be upload from the hosting machine)
     # extension of the targeted file, base path which will be the location of the target file,
